﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

namespace KuduRestClient
{
    class Program
    {
        public static async Task<int> Main(string[] args)
        {
            string siteId = "[...sitename...]";
            string packageId = "[...package id...]";

            if (args.Length >= 2)
            {
                siteId = args[0];
                packageId = args[1];
            }

            (string account, string password) = GetAccountInfo(siteId);

            if (string.IsNullOrEmpty(account) || string.IsNullOrEmpty(password))
            {
                Console.WriteLine("Account info is null");
                return 1;
            }

            KuduClient client = new KuduClient(siteId, account, password);

            ScmInfo scmInfo = await client.GetScmInfoAsync();
            Console.WriteLine(scmInfo.GitUrl);

            List<SiteExtensionItem> list = await client.GetInstalledSiteExtensionListAsync("[...extension name...]");

            if (list.Count >= 1)
            {
                SiteExtensionItem siteExtension = await client.GetInstalledSiteExtensionAsync(list[0].id);

                if (siteExtension != null)
                {
                    ProcessInfo mainProcessInfo = await client.GetMainSiteInfoAsync();
                    int oldProcessId = mainProcessInfo.id;

                    MessageResponse mr = await client.DeleteInstalledSiteExtensionAsync(siteExtension.id);
                    Console.WriteLine(mr.HttpResultCode + ": " + mr.Message);

                    Console.WriteLine("Restarting: ");
                    if (mr.HttpResultCode == System.Net.HttpStatusCode.OK)
                    {
                        await client.RestartMainSiteAsync();
                    }

                    while (true)
                    {
                        mainProcessInfo = await client.GetMainSiteInfoAsync();
                        if (mainProcessInfo.id == 0 || mainProcessInfo.id != oldProcessId)
                        {
                            break;
                        }

                        Console.Write(".");
                    }

                    Console.WriteLine();
                }
            }
            else
            {
                MessageResponse mr = await client.InstallSiteExtension(packageId);
                Console.WriteLine(mr.HttpResultCode + ": " + mr.Message);

                if (mr.HttpResultCode != System.Net.HttpStatusCode.OK)
                {
                    await RestartMainWebSite(client);
                }
            }

            // CommandOutput output = await client.RunCommandAsync("iisreset", ""); // Access denied
            // CommandOutput output = await client.RunCommandAsync("az", ""); // is not recognized as an internal or external command, operable program or batch file.
            // Console.WriteLine(output.Output + "\n" + output.Error);

            return 0;
        }

        private static async Task RestartMainWebSite(KuduClient client)
        {
            ProcessInfo mainProcessInfo = await client.GetMainSiteInfoAsync();
            int oldProcessId = mainProcessInfo.id;

            Console.WriteLine("Restarting: ");

            MessageResponse mr = await client.RestartMainSiteAsync();
            if (mr.HttpResultCode == System.Net.HttpStatusCode.OK)
            {
                return;
            }

            while (true)
            {
                mainProcessInfo = await client.GetMainSiteInfoAsync();
                if (mainProcessInfo.id != oldProcessId)
                {
                    break;
                }

                Console.Write(".");
            }
        }

        private static bool IfLogFiles(string name)
        {
            string ext = Path.GetExtension(name);

            switch (ext)
            {
                case ".xml":
                case ".log":
                case ".txt":
                    return true;
            }

            return false;
        }

        private static (string account, string password) GetAccountInfo(string siteId)
        {
            string[] lines = null;
            (string account, string password) result = (null, null);

            try
            {
                lines = File.ReadAllLines($@"c:\temp\azure.{siteId}.publish.txt");
                result.account = lines[0];
                result.password = lines[1];
            } catch { }

            return result;
        }
    }
}
